#!/usr/bin/perl -w
#$HeadURL: http://gruen.4g/svn/branches/stable-1.0/application/nylon-config/srv/www/cgi-bin/Config/BackupRestore.pm $ $LastChangedRevision: 926 $

=head1 Backup and Restore

This is a pseudo backend for backup and restore stuff:

=over 8

=item *

create config backup

=item *

create image backup (ToDo)

=item *

restore any killroy-file (ToDo)

=back

=cut

package Config::BackupRestore;
use strict;

=head1 SYNOPSIS

=over 8

=item my $br = new Config::BackupRestore();

=cut

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {
		SETTINGS => {
			     BackupPath => "/usr/backup/factory-settings.pkg",
			     FileList => "/etc/default/factory-settings.list",
			     },
		};
    bless($self,$class);
}

=pod

=item $br->set(option,value)

with options

=over 4

=item BackupPath

Name of final I<killroy> file
Default: /usr/backup/config.pkg

"-" means stdout

=item FileList

Path to file containing a list of paths to files
Default: /etc/default/configbackup.list

=item HUGE

wether or not /tmp needs to be resized

=back 4

=cut

sub set {
    my $self = shift;
    my $opt = shift;
    my $val = shift;
    my $options = ["BackupPath",
		   "FileList",
		   "HUGE",
		   ];
    # plausi
    do {
	$self->{_ERROR} = "option (1st) and value (2nd) missing";
	return 0;
    } unless defined $val;
    do {
	$self->{_ERORR} = "option (1st arg) missing";
	return 0;
    } unless defined $opt;
    do {
	$self->{_ERROR} = "unknown option: $opt";
	return 0;
    } unless grep {$opt eq $_} @$options;
    $self->{SETTINGS}->{$opt} = $val;
    return(1);
}

=pod

=item $br->make_config_backup([destination]);

constructs a I<killroy> backup of config files listed in the file
{SETTINGS}->{FileList}.

[destination] is optional, in which case {SETTINGS}->{BackupPath}
will be used.

A value of "-" means: STDOUT

=cut

sub make_config_backup {
    my $self = shift;
    my $dest = shift;
    if (defined $dest) {
	$self->{SETTINGS}->{BackupPath} = $dest;
    } else {
	$dest = $self->{SETTINGS}->{BackupPath};
    }
    do{
	read_file($self);
    } unless defined($self->{VARS}->{FileList});
    my $tmptar = "/tmp/data.tar.gz";
    if (fork) {
	wait;
    } else {
	open STDERR, ">/dev/null";
	exec("tar",
	     "-C", "/",
	     "-c", "-z",
	     "-f", $tmptar,
	     @{$self->{VARS}->{FileList}},
	    );
    }
    my $doit = "/tmp/doit";
    open(DOIT,">$doit") || do {
	$self->{_ERROR} = "can't create doit file \"$doit\"";
	return 0;
    };
    #-------------------------------------------------------------------------
    print DOIT <<EOF;
#!/bin/sh
# backup of config file listed in $self->{SETTINGS}->{FileList}
tar -x -O -f \$1 data.tar.gz | (
	cd /
	tar xvzf -
)
EOF
    #-------------------------------------------------------------------------
    close(DOIT);
    chmod(0555, $doit);
    system("/bin/gzip",$doit);
    system("/bin/tar",
	   "-C", "/tmp",
	   "-c", "-f", $dest,
	   "doit.gz", "data.tar.gz");
    unlink("$doit.gz","/tmp/data.tar.gz");
    return 1;
}

=pod

=item $br->read_file();

reads the file containing the backup list into an internal buffer.
Could be private

=cut

sub read_file {
    my $self = shift;
    my $file = $self->{SETTINGS}->{FileList};
    open CFG,$file || do {
	$self->{_ERROR} = "unable to load file list $file";
	return 0;
    };
    $self->{VARS}->{FileList} = [];
    while(<CFG>) {
	chomp;
	push @{$self->{VARS}->{FileList}},$_;
    }
    return 1;
}

=item $br->restore();

restore the backup / killroy file named in option C<BackupPath>
Resizes the /tmp FS if option C<HUGE> is set to a perl-TRUE value

=cut

sub restore {
    my $self = shift;
    grow_tmp($self);
    open STDERR, ">/dev/null";
    open OSTDOUT, "&>STDOUT";
    open STDOUT, ">/dev/null";
    system("/usr/sbin/install-upgrade",
	 $self->{SETTINGS}->{BackupPath}) || do {
	     $self->{_ERROR} = "upgrade/restore failed";
	     shrink_tmp($self);
	     return(0);
	 };
    shrink_tmp($self);
    open STDOUT,"&>OSTDOUT";
    return 1;
}

sub resize_tmp {
    my $size=shift;
    system("mount", "-o", "remount,size=" . $size . "M,", "/var");
}

sub grow_tmp {
    my $self = shift;
    my $size = tmpsize();
    my $huge = $self->{SETTINGS}->{HUGE};
    if ($huge && ($size < $huge)) {
	push @{$self->{SETTINGS}->{tmpsize}}, $size;
	resize_tmp($huge);
    }
}

sub shrink_tmp {
    my $self = shift;
    my $size = pop(@{$self->{SETTINGS}->{tmpsize}});
    resize_tmp($size) if ($size);
}

sub tmpsize {
    open DF, "df |";
    while(<DF>) {
	my ($dev,$size,$used,$avail,$perc,$mpoint) =  split(/\s+/);
	if ($dev eq "tmpfs") {
	    close DF;
	    return $size / 1024;
	}
    }
    return -1;
}

=back

=head1 Killroy File Format

A I<killroy package> is an uncompressed tar archive that contains whatever
is desirable, B<but allways> a file I<./doit.gz>.
I<doit> is usually a shell file and is executed by C</usr/sbin/install-upgrade>
whith the first argument being the path to the I<killroy package>.

This should be flexible enough for any backup/restore/upgrade task

=head2 The Name

Umm...
the first dummy package to test the mechanism with was S<"killroy.pkg">, which
did nothing but install a file "KILLROY" into root's home directory containing
"was here :-)"

=cut

1;
### Local Variables:
### mode: cperl
### End:
