#!/usr/bin/perl -w

package Config::Dnsmasq;

use Exporter;
use CWeb::Tools;

our @ISA =  "Exporter";
our @EXPORT_OK = qw{ read write };

#-------------------------------------------------------------------
# generic "new" method for config file handlers
sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $cfg_file = shift;
  $cfg_file = "/etc/dnsmasq.conf" unless defined $cfg_file;
  my $self = { _CONFIG_FILE => $cfg_file, };
  my $meta = {};
  bless($self,$class);
}
#-------------------------------------------------------------------

sub read {
	my $self = shift;
	# config-file from object,- unless called as module-function
	my $file = ref($self) eq __PACKAGE__ ? $self->{_CONFIG_FILE} : shift;
	my $tool = new CWeb::Tools;
	my $vars = {};
	
	open(VARS,$file) || warn "can't open config file $file\n";
	$lineno=0;
	
	while (<VARS>) 
	{
		$meta->{$lineno}->{"LINE"} = $_;
		if (/^#/) {
		}
		elsif (/\s*([^=\s]+)\s*=\s*(\S*)(\s*\#?.*)/s) {
			# matched name=value,value2,value3,... parameter
			my ($name, $value) = ($1,$2);
			$meta->{$lineno}->{"COMMENT"} = $3;
			@v = split /,/,$value;
			my $item = {};
			
			# fake unique names for dhcp-options
			if ($name =~ /dhcp-option/) {
				$name = "dhcp-option".$v[0];
			}
			$meta->{$lineno}->{"NAME"} = $name;
			
			# find number of items with the same name
			for ($i=0; defined $vars->{$name}->{$i}; $i++) {
			}
			# convert array to hash
			$j=0;
			foreach (@v) {
				$item->{$j++} = $_;
			}
			$meta->{$lineno}->{"PARSED"} = $item;
			$meta->{$lineno}->{"INDEX"} = $i;
			$vars->{$name}->{$i} = $item;
		}
		elsif (/\s*(\S+)(\s*.*)/) {
			# single name statements
			$meta->{$lineno}->{"COMMENT"} = $2;
			$meta->{$lineno}->{"NAME"} = $1;
			$vars->{$1} = 1;
		}
		$lineno++;
	}
	$vars;
}

sub write {
	my $self = shift;
	# config-file from object,- unless called as module-function
	my $conffile = ref($self) eq __PACKAGE__ ? $self->{_CONFIG_FILE} : shift;
	my $vars = shift;

	my ($newfile) = $conffile;
	$newfile =~ s+.*/++;
	$newfile = "/tmp/$newfile";
	open(NEW,">$newfile") || die "can't write to $newfile\n";

	my $item, $key, $line, $lineno;
	for ($lineno=0; defined $meta->{$lineno}; $lineno++) {
		$line = $meta->{$lineno};
		if (defined $line->{"PARSED"}) {
			$item = $line->{"PARSED"};
			print_item( $line->{"NAME"}, $item, $line->{"COMMENT"} );
		} else {
			print NEW $line->{"LINE"};
		}
		# single word statements are not written yet
		$item->{"WRITTEN"}=1;
	}
	# add new elements at the end
	foreach ( keys %$vars) {
		$key = $_;
		$item = $vars->{$key};
		# every name has an index, write all remaining
		foreach ( sort keys %$item) {
			print_item( $key, $item->{$_}, "\n" );
		}
  }
  close NEW;
  $_ = `cat $newfile > $conffile`;
}

sub print_item {
	my $name = shift;
	my $item = shift;
	my $comment = shift;
	my $value = "";
	my $empty = 0;
	
	return if $item->{"WRITTEN"};
	
	# special handling for dhcp options
	if ( $name =~ /dhcp-option/) {
		$name = "dhcp-option";
	}
	# write all parts, seperated with ,
	foreach ( sort keys %$item) {
		$empty = 1 if !$item->{$_};
		$value = $value . $item->{$_} . ",";
	}
	$value =~ s/,$//;
	
	if (!$empty && $value) {
		print NEW $name . "=" . $value . $comment;
		return 1;
	}
	return 0;
}

#Local Variables:
#mode: cperl
#End:
