#!/usr/bin/perl -w
#$Header: /data/cvs/mtx/application/web-config/var/www/cgi-bin/Config/IFace.pm,v 1.1 2004/05/26 12:03:19 br1 Exp $

package Config::Security;

use strict;

use Exporter;

use Config::IFace;
use CWeb::Tools;

our @ISA = "Exporter";
our @EXPORT_OK = qw{ sec_mode ssid_set values_get write  };

# hostapd needs an ssid. To set the ssid use method 
# "$object->ssid_set( <the new ssid as string> )"
# if necessary hostapd gets reloaded.

# call "$object->sec_mode()" to get current security mode. String
# NULL is returned in case of no encryption or absnet hostapd.

#-------------------------------------------------------------------

sub new {

	my $proto		= shift;
 	my $class		= ref($proto) || $proto;

	my $self		= {
				'iface0'		=> 'ath0',
				'cfg_file' 		=> '/etc/hostapd.conf',
				'init_file'		=> '/etc/init.d/hostapd',
				'hapd_init_on'	=> 'update-rc.d hostapd defaults > /dev/null 2>&1',
				'hapd_init_off'	=> 'update-rc.d -f hostapd remove > /dev/null 2>&1'
	} ;
	$self->{'acl_allow'}		= '/etc/network/acl/' . $self->{'iface0'} . '.allow' ;
	$self->{'acl_deny'}			= '/etc/network/acl/' . $self->{'iface0'} . '.deny' ;
	$self->{'ifdown'}			= 'ifdown ' . $self->{'iface0'} . ' > /dev/null 2>&1' ;
	$self->{'ifup'}				= 'ifup ' . $self->{'iface0'} . ' > /dev/null 2>&1' ;
	bless( $self, $class ) ;
	$self->{ 'wpa' } 	= $self->read_wpa_conf_file() ;
	$self->{ 'wep' }	= $self->read_wep_params() ;
	$self->{ 'acl' }	= '' ;
	$self->{ 'acl_a' }	= $self->read_acl_file( $self->{ 'acl_allow' } ) ;
	$self->{ 'acl_d' }	= $self->read_acl_file( $self->{ 'acl_deny' } ) ;
	$self->{ 'misc' }	= {} ;
	return $self ;
}

#-------------------------------------------------------------------

sub mac_edit {

	my $self 		= shift;

	my $cmd			= shift || return '' ;
	my $mac			= shift || return '' ;

	my $list		= '' ;
	my $i 			= 0 ;

	$self->{ 'misc' }->{ 'changed' } = 0 ;

	if ( $cmd eq 'addmacallow' ) {
		$list = $self->read_acl_file( $self->{ 'acl_allow' } ) ;
		push @{$list}, $mac ;
		$self->write_acl_file( $self->{ 'acl_allow' } , $list ) ;
		$self->{ 'misc' }->{ 'changed' } = 1 ;
	}
	elsif ( $cmd eq 'addmacdeny' ) {
		$list = $self->read_acl_file( $self->{ 'acl_deny' } ) ;
		push @{$list}, $mac ;
		$self->write_acl_file( $self->{ 'acl_deny' } , $list ) ;
		$self->{ 'misc' }->{ 'changed' } = 1 ;
	}

	elsif ( $cmd eq 'delmacallow' ) {
		$i = 0 ;
		$list = $self->read_acl_file( $self->{ 'acl_allow' } ) ;
		while ( 1 ) {
			$list->[$i] || last ;
			if ( $list->[$i] eq $mac ) { 
				delete $list->[$i] ;  
				$self->{ 'misc' }->{ 'changed' } = 1 ;
			}
			$i++ ;
		}
		if ( $self->{ 'misc' }->{ 'changed' } ) {
			$self->write_acl_file( $self->{ 'acl_allow' } , $list ) ;
		}
	}
	elsif ( $cmd eq 'delmacdeny' ) {
		$i = 0 ;
		$list = $self->read_acl_file( $self->{ 'acl_deny' } ) ;
		while ( 1 ) {
			$list->[$i] || last ;
			if ( $list->[$i] eq $mac ) { 
				delete $list->[$i] ; 
				$self->{ 'misc' }->{ 'changed' } = 1 ;
			}
			$i++ ;
		}
		if ( $self->{ 'misc' }->{ 'changed' } ) {
			$self->write_acl_file( $self->{ 'acl_deny' } , $list ) ;
		}
	}

	if ( $self->{ 'misc' }->{ 'changed' } ) {
		`$self->{ 'ifdown' }` ;
		`$self->{ 'ifup' }` ;
	}
}

#-------------------------------------------------------------------

sub read_acl_file {
	my $self 		= shift;
	my $file		= shift || return '' ;
	my $list = [];

	unless( open( ACL, "< $file" ) ) { 
		return $list ;
	}
	while (<ACL>) {
		# clean up
		s/#.*//;
		s/^\s+//;
		s/\s+$//;
		chomp();
		next unless length;

		push @{$list}, $_ ;
	}
	close ACL;
	return $list ;
}

#-------------------------------------------------------------------



sub read_wep_params {

	my $self 		= shift ;

	my $conf_if 	= new Config::IFace ;							
	my $tmp 		= $conf_if->read() ;
	my $int			= $tmp->{ 'Interface' }->{$self->{'iface0'}} ;

	my $wep			= {
				'keymode'		=> $int->{ 'WIRELESS_KEYMODE' } || 'off' ,
				'default_key'	=> $int->{ 'WIRELESS_DEFAULTKEY' } || '1' ,
				'key1' 			=> $int->{ 'WIRELESS_KEY1' } || '' ,
				'key2' 			=> $int->{ 'WIRELESS_KEY2' } || '' ,
				'key3' 			=> $int->{ 'WIRELESS_KEY3' } || '' ,
				'key4'	 		=> $int->{ 'WIRELESS_KEY4' } || '',
				'acl'			=> $int->{ 'WIRELESS_ACL' } || 'open'
	} ;

	return $wep ;
}

#-------------------------------------------------------------------
# open the hostap-daemon config file and get the
# pieces we are interestet in

sub read_wpa_conf_file {

	my $self 		= shift;

	open( CFG, "< $self->{ 'cfg_file' }" ) || 
			warn "can not open config file $self->{ 'cfg_file' }\n";

	my $file_params = $self->values_init() ;

	while (<CFG>) {

		# clean up
		s/#.*//;
		s/^\s+//;
		s/\s+$//;
		chomp();
		next unless length;

		s/=(.*)//;

		if ( defined $file_params->{$_} ) {
			$file_params->{$_} = $1;
		}
	}

	close CFG;

	return $file_params ;
}

#-------------------------------------------------------------------

sub sec_mode {

	my $self 		= shift ;

	my $param		= shift || 0 ;
	my $value		= shift || 0 ;

	unless ( $param ) {
		if ( $self->wpa_service() ) {
			return 'WPA1' if $self->{ 'wpa' }->{ 'wpa' } eq '1';
			return 'WPA2' if $self->{ 'wpa' }->{ 'wpa' } eq '2';
			return 'WPA12' if $self->{ 'wpa' }->{ 'wpa' } eq '3';
		} 
		return 'WEP' if $self->{ 'wep' }->{ 'keymode' } eq 'on';
		return 'NULL';
	}

	# no encryption
	if ( $param eq 'off' ) {
		$self->{ 'wep' }->{ 'keymode' } = 'off open' ;
		$self->{ 'wpa' }->{ 'wpa' } = '0' ;
	}

	# WEP encryption
	elsif ( $param eq 'wep' ) {
		$self->{ 'wep' }->{ 'keymode' } = 'on' ;
		$self->{ 'wpa' }->{ 'wpa' } = '0' ;
		$self->{ 'wep' }->{ 'key1' } = $value->{ 'key1' } ;
		$self->{ 'wep' }->{ 'key2' } = $value->{ 'key2' } ;
		$self->{ 'wep' }->{ 'key3' } = $value->{ 'key3' } ;
		$self->{ 'wep' }->{ 'key4' } = $value->{ 'key4' } ;
		$self->{ 'wep' }->{ 'default_key' } = $value->{ 'default' } ;
	}

	# WPA encryption)
	elsif ( $param eq 'wpa' ) {
		$self->{ 'wep' }->{ 'keymode' } = 'off' ;
		$self->{ 'wpa' }->{ 'wpa' } = $value ;
	}
	elsif ( $param eq 'wpa_psk' ) {
		$self->{ 'wpa' }->{ 'wpa_psk' } = $value ;
		$self->{ 'wpa' }->{ 'wpa_passphrase' } = 0 ;
	}
	elsif ( $param eq 'wpa_phrase' ) {
		$self->{ 'wpa' }->{ 'wpa_psk' } = 0 ;
		$self->{ 'wpa' }->{ 'wpa_passphrase' } = $value ;
	}

	# read ssid from interface and set for hostapd
	elsif ( $param eq 'ssid' ) {
		my $conf_if = new Config::IFace;							
		my $int = $conf_if->read();
		$self->{ 'wpa' }->{ 'ssid' } = $int->{"Interface"}->{$self->{'iface0'}}->{"wireless_essid"} ;
	}

	# set ACL (mac filter)
	elsif ( $param eq 'acl' ) {
		if ( $value eq '1' ) {
			$self->{ 'acl' } = 'allow' ;
		}
		elsif( $value eq '2' ) {
			$self->{ 'acl' } = 'deny' ;
		}
		else {
			$self->{ 'acl' } = 'open' ;
		}
	}
}

#-------------------------------------------------------------------

sub ssid_set {

	my $self 		= shift;

	my $ssid		= shift || return 'value_error' ;

	$self->{ 'misc' }->{ 'changed' } = 0 ;
	$self->{ 'wpa' }->{ 'ssid' } = $ssid ;

	$self->write_wpa_conf_file( $self->{ 'wpa' } ) ;

	if ( $self->{ 'misc' }->{ 'changed' } ) {
		if ( $self->{ 'wpa' }->{ 'wpa' } > 0 ) {
			$self->wpa_service( 'restart' );
		}
	}
}

#-------------------------------------------------------------------

sub values_get {

	my $self 		= shift ;

	my $sec_params	= {
				'wpa'		=> $self->read_wpa_conf_file() ,
				'wep'		=> $self->read_wep_params(),
				'acl_a'		=> $self->read_acl_file( $self->{ 'acl_allow' } ) ,
				'acl_d'		=> $self->read_acl_file( $self->{ 'acl_deny' } ) ,
				'MAC_ALLOW'	=> { 'SIZE'		=> 0 },
				'MAC_DENY'	=> { 'SIZE'		=> 0 }
	} ;

	foreach ( @{$sec_params->{'acl_a'}} ) {
		$sec_params->{'MAC_ALLOW'}->{'SIZE'}++ ;
		$sec_params->{'MAC_ALLOW'}->{$sec_params->{'MAC_ALLOW'}->{'SIZE'}} =
			{ 'mac'		=> $_ } ;
	}
	foreach ( @{$sec_params->{'acl_d'}} ) {
		$sec_params->{'MAC_DENY'}->{'SIZE'}++ ;
		$sec_params->{'MAC_DENY'}->{$sec_params->{'MAC_DENY'}->{'SIZE'}} =
			{ 'mac'		=> $_ } ;
	}

	# for wpa use either a passphrase or a pre-shared-key
	if ( $sec_params->{ 'wpa' }->{ 'wpa_passphrase' } ) {
			$sec_params->{ 'misc' }->{ 'secret' } = $sec_params->{ 'wpa' }->{ 'wpa_passphrase' } ;
	}
	else {
			$sec_params->{ 'misc' }->{ 'secret' } = $sec_params->{ 'wpa' }->{ 'wpa_psk' } ;
	}

	$sec_params->{ 'acl' } = $sec_params->{ 'wep' }->{ 'acl' } ;

	return $sec_params ;
}

#-------------------------------------------------------------------

sub values_init {

	my $self 		= shift;

	# init the param table with zeros, so later we can verify
	# witch params did not get touched at all.
	my $wpa_params = {
			'wpa'				=> 0,	# WPA main mode (1|2|1+2)
			'wpa_passphrase'	=> 0,	# either a passphrase
			'wpa_psk'			=> 0,	# or a pre shared key, not both.
			'ssid'				=> 0
	};

	return $wpa_params;
}

#-------------------------------------------------------------------

sub wep_generate  {
	my $self 		= shift;
	my $phrase		= shift || return '' ;

	#open (TMP, "> /tmp/keys") ;
	my @keys = `nwepgen $phrase 13` ;
	foreach ( @keys ) {
		chomp() ;
		s/://g;
	}
	return @keys ;
}

#-------------------------------------------------------------------

sub wpa_service {

	# start and stop wpa serverice
	# commands are: start, stop, restart, status

	my $self 		= shift;

	my $cmd			= shift || 0;

	unless ( $cmd ) {
		if ( `ps ax | grep hostapd | grep -v grep` ) {
			return 'running';
		}
		return 0;
	}

	my $tool 		= new CWeb::Tools;
	my $cmds		= [];

	if ( $cmd eq "start" ) {
		push( @{$cmds}, [ $self->{'init_file'}, "start" ] );
		$tool->background_init($cmds);
		return "start";
	}
	elsif ( $cmd eq "stop" ) {
		push( @{$cmds}, [ $self->{'init_file'}, "stop" ] );
		$tool->background_init($cmds);
		return "stop";
	}
	elsif ( $cmd eq "restart" ) {
		push( @{$cmds}, [ $self->{'init_file'}, "stop" ] );
		push( @{$cmds}, [ $self->{'init_file'}, "start" ] );
		$tool->background_init($cmds);
		return "restart";
	}
	return 0;
}

#-------------------------------------------------------------------

sub write {

	my $self 		= shift;

	$self->{ 'misc' }->{ 'changed' } = 0 ;

	$self->write_acl( $self->{ 'acl' } ) ;
	$self->write_wpa_conf_file( $self->{ 'wpa' } ) ;
	$self->write_wep_params( $self->{ 'wep' } ) ;

	if ( $self->{ 'misc' }->{ 'changed' } ) {

		if ( $self->{ 'wpa' }->{ 'wpa' } > 0 ) {
			`$self->{ 'hapd_init_on' }` ; ;
			`$self->{ 'ifdown' }` ;
			`$self->{ 'ifup' }` ;
			$self->wpa_service( 'restart' ) ;
		}
		else {
			$self->wpa_service( 'stop' ) ;
			`$self->{ 'hapd_init_off' }` ; ;
			`$self->{ 'ifdown' }` ;
			`$self->{ 'ifup' }` ;
		}
	}
}

#-------------------------------------------------------------------

sub write_acl {

	my $self 		= shift ;

	my $acl			= shift ;

	my $conf_if 	= new Config::IFace ;							
	my $int 		= $conf_if->read() ;

	if ( $int->{ 'Interface' }->{$self->{'iface0'}}->{ 'WIRELESS_ACL' } ne $acl ) {
		$int->{ 'Interface' }->{$self->{'iface0'}}->{ 'WIRELESS_ACL' } = $acl ;
		$conf_if->write( $int ) ;

		$self->{ 'misc' }->{ 'changed' } = 1 ;
	}
}

#-------------------------------------------------------------------

sub write_acl_file {

	my $self 		= shift;

	my $file		= shift || return '' ;
	my $mac_list	= shift || return '' ;

	unless( open( ACL, "> $file" ) ) { 
		warn "can not open ACL file $file\n";
		return '' ;
	}

	foreach ( @{$mac_list} ) {

		print ACL "$_\n" ;
	}

	close ACL ;

	return ( scalar @{$mac_list} ) ;
}

#-------------------------------------------------------------------

sub write_wep_params {

	my $self 		= shift ;

	my $sec_val_new	= shift ;

	unless ( $sec_val_new->{ 'keymode' } ) {
					$sec_val_new->{ 'keymode' } = 'off' ; }
	unless ( $sec_val_new->{ 'default_key' } ) {
					$sec_val_new->{ 'default_key' } = '1' ; }

	my $sec_val_old = $self->read_wep_params() ;

	my @change = ();
	foreach ( keys %{$sec_val_new} ) {
		if ( $sec_val_old->{$_} ne $sec_val_new->{$_} ) { push @change, $_ ; }
	}

	if ( ( scalar @change ) > 0) {

		my $conf_if 	= new Config::IFace ;							
		my $int 		= $conf_if->read() ;

		$int->{ 'Interface' }->{$self->{'iface0'}}->{ 'WIRELESS_KEYMODE' } = $sec_val_new->{ 'keymode' } ;
		$int->{ 'Interface' }->{$self->{'iface0'}}->{ 'WIRELESS_DEFAULTKEY' } = $sec_val_new->{ 'default_key' } ;
		$int->{ 'Interface' }->{$self->{'iface0'}}->{ 'WIRELESS_KEY1' } = $sec_val_new->{ 'key1' } ;
		$int->{ 'Interface' }->{$self->{'iface0'}}->{ 'WIRELESS_KEY2' } = $sec_val_new->{ 'key2' } ;
		$int->{ 'Interface' }->{$self->{'iface0'}}->{ 'WIRELESS_KEY3' } = $sec_val_new->{ 'key3' } ;
		$int->{ 'Interface' }->{$self->{'iface0'}}->{ 'WIRELESS_KEY4' } = $sec_val_new->{ 'key4' } ;

		$conf_if->write( $int ) ;

		$self->{ 'misc' }->{ 'changed' } = 1 ;
	}
}

#-------------------------------------------------------------------

sub write_wpa_conf_file {

	my $self 		= shift;

	my $sec_val_new	= shift;

	# Set some sensible default values if necessary
	$sec_val_new->{'wpa'} = 3 unless (($sec_val_new->{'wpa'} >= 0) and ($sec_val_new->{'wpa'} <= 3));
	(not $sec_val_new->{'wpa_passphrase'} and not $sec_val_new->{'wpa_psk'}) && do {
		$sec_val_new->{'wpa_passphrase'} = 'do_not_tell_anybody'; };
	$sec_val_new->{'ssid'} = '4G-Access-Cube' unless $sec_val_new->{'ssid'};

	# Check if the config needs to get rewritten
	my $sec_val_old = $self->read_wpa_conf_file();	
	my @change = ();
	foreach ( keys %{$sec_val_new} ) {
		if ( $sec_val_old->{$_} ne $sec_val_new->{$_} ) { push @change, $_ ; }
	} 

	if ((scalar @change) > 0) {
		unless (open( CFG, "< $self->{ 'cfg_file' }")) { 
			warn "can not open config file $self->{'cfg_file'}\n";
			return 'config_read_error';
		}
		my @cfg_content = <CFG>;
		close CFG;

		my $pos = 0;
		my $match = '';
		while ($cfg_content[$pos]) {
			foreach (@change) {
				$match = $_ . '=';
				if ($cfg_content[$pos] =~ m/$match(.*)/) {
					$cfg_content[$pos] =~ s/$1/$sec_val_new->{$_}/;
					if (($_ =~ m/wpa_/) and 
							($sec_val_new->{$_} eq 0) and not
							($cfg_content[$pos] =~ /^#/)) {

						$cfg_content[$pos] = '#' . $cfg_content[$pos];
					}
					if (($_ =~ m/wpa_/) and 
							($sec_val_new->{$_} ne 0) and
							($cfg_content[$pos] =~ /^#/)) {
						$cfg_content[$pos] =~ s/^#\s*//;
					}						
				}
			}
			$pos++;
		}
		unless (open( CFG, "> $self->{ 'cfg_file' }")) { 
			warn "can not open config file $self->{ 'cfg_file' }\n";
			return 'config_write_error';
		}
		print CFG @cfg_content;
		close CFG;

		$self->{'misc'}->{'changed'} = 1 ;

		return 'config_written';
	} else {	
		return 0;
	}
}

#-------------------------------------------------------------------

#Local Variables:
#mode: cperl
#End:
